/******************************************************************************
*  Copyright 2017 - 2018, Opulinks Technology Ltd.
*  ----------------------------------------------------------------------------
*  Statement:
*  ----------
*  This software is protected by Copyright and the information contained
*  herein is confidential. The software may not be copied and the information
*  contained herein may not be used or disclosed except with the written
*  permission of Opulinks Technology Ltd. (C) 2018
******************************************************************************/

/******************************************************************************
*  Filename:
*  ---------
*  main_patch.c
*
*  Project:
*  --------
*  OPL1000 Project - the main patch implement file
*
*  Description:
*  ------------
*  This implement file is include the main patch function and api.
*
*  Author:
*  -------
*  SH SDK
*
******************************************************************************/
/***********************
Head Block of The File
***********************/
// Sec 0: Comment block of the file
/******************************************************************************
*  Test code brief
*  These examples show how to configure uart settings and use uart driver APIs.
*
*  uart_int_test() is an example that read and write data on UART0, and send
*  the rx data to thread when the rx data is received by UART0 interrupt.
*  - port: UART0
*  - interrupt: on
*  - flow control: off
*  - pin assignment: tx(io2), rx(io3)
*
*  uart_echo_test() is an example that read and write data on UART1, and the
*  hardware flow control is turning on.
*  - port: UART1
*  - interrupt: off
*  - flow control: on
*  - pin assignment: tx(io4), rx(io5), rts(io6), cts(io7)
******************************************************************************/


// Sec 1: Include File
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include "sys_init.h"
#include "sys_init_patch.h"
#include "hal_system.h"
#include "mw_fim.h"
#include "cmsis_os.h"
#include "sys_os_config.h"
#include "hal_auxadc.h"
#include "hal_auxadc_patch.h"
#include "Hal_pinmux_uart.h"
#include "Hal_pinmux_gpio.h"
#include "hal_pin.h"
#include "hal_pin_def.h"
#include "hal_pin_config_project.h"
#include "at_cmd_common_patch.h"
#include "hal_dbg_uart.h"
#include "hal_vic.h"
#include "boot_sequence.h"
#include "http_request.h"
#include "util.h"

//
// WiZPܦZ
//
#define SENSOR_TO_BOTTOM 23 // cm

//
// wİϤjp
//
#define MAX_DATA_SIZE 16

//
// ]wؼФ찪
//
float TargetWaterLevelL = 10.0;
float TargetWaterLevelH = 11.0;

// Sec 2: Constant Definitions, Imported Symbols, miscellaneous
// the number of elements in the message queue
#define APP_MESSAGE_Q_SIZE  16

// the event type of message
#define APP_EVENT_UART_RX   0x01


/********************************************
Declaration of data structure
********************************************/
// Sec 3: structure, uniou, enum, linked list
// the content of message queue
typedef struct
{
    uint32_t ulEvent;
    uint32_t ulData;
} S_MessageQ;


/********************************************
Declaration of Global Variables & Functions
********************************************/
// Sec 4: declaration of global variable


// Sec 5: declaration of global function prototype
typedef void (*T_Main_AppInit_fp)(void);
extern T_Main_AppInit_fp Main_AppInit;


/***************************************************
Declaration of static Global Variables & Functions
***************************************************/
// Sec 6: declaration of static global variable
static osThreadId g_tAppThread_1;
static osThreadId g_tAppThread_2;
static osMessageQId g_tAppMessageQ;
static osPoolId g_tAppMemPoolId;

static E_IO01_UART_MODE g_eAppIO01UartMode;


// Sec 7: declaration of static function prototype
void __Patch_EntryPoint(void) __attribute__((section("ENTRY_POINT")));
void __Patch_EntryPoint(void) __attribute__((used));
static void Main_PinMuxUpdate(void);
static void Main_FlashLayoutUpdate(void);
void Main_AppInit_patch(void);
static void Main_AppThread_1(void *argu);
static void Main_AppMessageQSend(uint32_t ulData);
static void init_thread(void);


static void Main_MiscModulesInit(void);

static void Main_MiscDriverConfigSetup(void);
static void Main_AtUartDbgUartSwitch(void);

static void Main_ApsUartRxDectecConfig(void);
static void Main_ApsUartRxDectecCb(E_GpioIdx_t tGpioIdx);


//
// WiPZ (:MM)
//
int Distance = 0;

//
// 찪 (:CM)
//
float WaterLevel = 0.0;

//
// YӧCĵo͡AܼƭȬ 5
//
uint8_t FlagAlert = 0;

/***********
C Functions
***********/
// Sec 8: C Functions

/*************************************************************************
* FUNCTION:
*   __Patch_EntryPoint
*
* DESCRIPTION:
*   the entry point of SW patch
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
void __Patch_EntryPoint(void)
{
    // don't remove this code
    SysInit_EntryPoint();
#ifdef SWITCH_TO_32K_RC
    // Uncomment this function when the device is without 32k XTAL.
    Sys_SwitchTo32kRC();
#endif 
    
    // update the pin mux
    Hal_SysPinMuxAppInit = Main_PinMuxUpdate;
    
    // update the flash layout
    MwFim_FlashLayoutUpdate = Main_FlashLayoutUpdate;

    // the initial of driver part for cold and warm boot
    Sys_MiscModulesInit = Main_MiscModulesInit;
    Sys_MiscDriverConfigSetup = Main_MiscDriverConfigSetup;

    // update the switch AT UART / dbg UART function
    at_cmd_switch_uart1_dbguart = Main_AtUartDbgUartSwitch;
    
    Sys_SetUnsuedSramEndBound(0x440000);
    // application init
    Sys_AppInit = Main_AppInit_patch;
}

/*************************************************************************
* FUNCTION:
*   Main_PinMuxUpdate
*
* DESCRIPTION:
*   update the pin-mux setting
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_PinMuxUpdate(void)
{
    Hal_Pin_ConfigSet(0, HAL_PIN_TYPE_IO_0, HAL_PIN_DRIVING_IO_0);
    Hal_Pin_ConfigSet(1, HAL_PIN_TYPE_IO_1, HAL_PIN_DRIVING_IO_1);
    Hal_Pin_ConfigSet(2, HAL_PIN_TYPE_IO_2, HAL_PIN_DRIVING_IO_2);
    Hal_Pin_ConfigSet(3, HAL_PIN_TYPE_IO_3, HAL_PIN_DRIVING_IO_3);
    Hal_Pin_ConfigSet(4, HAL_PIN_TYPE_IO_4, HAL_PIN_DRIVING_IO_4);
    Hal_Pin_ConfigSet(5, HAL_PIN_TYPE_IO_5, HAL_PIN_DRIVING_IO_5);
    Hal_Pin_ConfigSet(6, HAL_PIN_TYPE_IO_6, HAL_PIN_DRIVING_IO_6);
    Hal_Pin_ConfigSet(7, HAL_PIN_TYPE_IO_7, HAL_PIN_DRIVING_IO_7);
    Hal_Pin_ConfigSet(8, HAL_PIN_TYPE_IO_8, HAL_PIN_DRIVING_IO_8);
    Hal_Pin_ConfigSet(9, HAL_PIN_TYPE_IO_9, HAL_PIN_DRIVING_IO_9);
    Hal_Pin_ConfigSet(10, HAL_PIN_TYPE_IO_10, HAL_PIN_DRIVING_IO_10);
    Hal_Pin_ConfigSet(11, HAL_PIN_TYPE_IO_11, HAL_PIN_DRIVING_IO_11);
    Hal_Pin_ConfigSet(12, HAL_PIN_TYPE_IO_12, HAL_PIN_DRIVING_IO_12);
    Hal_Pin_ConfigSet(13, HAL_PIN_TYPE_IO_13, HAL_PIN_DRIVING_IO_13);
    Hal_Pin_ConfigSet(14, HAL_PIN_TYPE_IO_14, HAL_PIN_DRIVING_IO_14);
    Hal_Pin_ConfigSet(15, HAL_PIN_TYPE_IO_15, HAL_PIN_DRIVING_IO_15);
    Hal_Pin_ConfigSet(16, HAL_PIN_TYPE_IO_16, HAL_PIN_DRIVING_IO_16);
    Hal_Pin_ConfigSet(17, HAL_PIN_TYPE_IO_17, HAL_PIN_DRIVING_IO_17);
    Hal_Pin_ConfigSet(18, HAL_PIN_TYPE_IO_18, HAL_PIN_DRIVING_IO_18);
    Hal_Pin_ConfigSet(19, HAL_PIN_TYPE_IO_19, HAL_PIN_DRIVING_IO_19);
    Hal_Pin_ConfigSet(20, HAL_PIN_TYPE_IO_20, HAL_PIN_DRIVING_IO_20);
    Hal_Pin_ConfigSet(21, HAL_PIN_TYPE_IO_21, HAL_PIN_DRIVING_IO_21);
    Hal_Pin_ConfigSet(22, HAL_PIN_TYPE_IO_22, HAL_PIN_DRIVING_IO_22);
    Hal_Pin_ConfigSet(23, HAL_PIN_TYPE_IO_23, HAL_PIN_DRIVING_IO_23);
    at_io01_uart_mode_set(HAL_PIN_0_1_UART_MODE);
}

/*************************************************************************
* FUNCTION:
*   Main_FlashLayoutUpdate
*
* DESCRIPTION:
*   update the flash layout
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_FlashLayoutUpdate(void)
{
    // update here
}

/*************************************************************************
* FUNCTION:
*   Main_MiscModulesInit
*
* DESCRIPTION:
*   the initial of driver part for cold and warm boot
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_MiscModulesInit(void)
{
     
}

/*************************************************************************
* FUNCTION:
*   Main_MiscDriverConfigSetup
*
* DESCRIPTION:
*   the initial of driver part for cold and warm boot
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_MiscDriverConfigSetup(void)
{
    //Hal_Wdt_Stop();   //disable watchdog here.

    // IO 1 : detect the GPIO high level if APS UART Rx is connected to another UART Tx port.
    // cold boot
    if (0 == Boot_CheckWarmBoot())
    {
        Hal_DbgUart_RxIntEn(0);
        
        if (HAL_PIN_TYPE_IO_1 == PIN_TYPE_UART_APS_RX)
        {
            Main_ApsUartRxDectecConfig();
        }
    }
}

/*************************************************************************
* FUNCTION:
*   Main_AtUartDbgUartSwitch
*
* DESCRIPTION:
*   switch the UART1 and dbg UART
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_AtUartDbgUartSwitch(void)
{
    if (g_eAppIO01UartMode == IO01_UART_MODE_AT)
    {
        Hal_Pin_ConfigSet(0, PIN_TYPE_UART_APS_TX, PIN_DRIVING_FLOAT);
        Hal_Pin_ConfigSet(1, PIN_TYPE_UART_APS_RX, PIN_DRIVING_LOW);

        Hal_Pin_ConfigSet(8, PIN_TYPE_UART1_TX, PIN_DRIVING_FLOAT);
        Hal_Pin_ConfigSet(9, PIN_TYPE_UART1_RX, PIN_DRIVING_HIGH);

        Hal_DbgUart_RxIntEn(1);
    }
    else
    {
        Hal_DbgUart_RxIntEn(0);

        Hal_Pin_ConfigSet(0, PIN_TYPE_UART1_TX, PIN_DRIVING_FLOAT);
        Hal_Pin_ConfigSet(1, PIN_TYPE_UART1_RX, PIN_DRIVING_LOW);
        
        Hal_Pin_ConfigSet(8, PIN_TYPE_UART_APS_TX, PIN_DRIVING_FLOAT);
        Hal_Pin_ConfigSet(9, PIN_TYPE_UART_APS_RX, PIN_DRIVING_HIGH);
    }
    
    g_eAppIO01UartMode = (E_IO01_UART_MODE)!g_eAppIO01UartMode;
}

/*************************************************************************
* FUNCTION:
*   App_Pin_InitConfig
*
* DESCRIPTION:
*   init the pin assignment
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
void App_Pin_InitConfig(void)
{
    Hal_PinMux_Uart_Init(&OPL1000_periph.uart[0]);
/*      uint8_t i; 
      for (i=0;i<OPL1000_periph.uart_num;i++)
      {
            Hal_PinMux_Uart_Init(&OPL1000_periph.uart[i]);
    }
*/
}

/*************************************************************************
* FUNCTION:
*   Main_AppInit_patch
*
* DESCRIPTION:
*   the initial of application
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
void Main_AppInit_patch(void)
{
    // init the pin assignment
    App_Pin_InitConfig();

    // do the gpio interrupt init
    //gpio_int_init();

    // Indicate UART test example is running 
    printf("UART Running \n");
    
    // do the thread initialization
    init_thread();
}

//
// qWiPŪZƭ (:MM)
//
// WiP UART X榡G
//   Offset             
//   0        Header        0xff
//   1        Z8
//   2        ZC8
//   3        Check Sum
//
int readUltraSonicUart(int uartIdx, int uartTx) {
  uint8_t buff[MAX_DATA_SIZE];
  int buff_index = 0;
  uint32_t data32 = 0;
  E_GpioLevel_t eGpioLevel;

  //
  // WiPoe@ӧCqTAĲo}lZC
  //
  digitalWrite(uartTx, GPIO_LEVEL_LOW);

  delay(5);

  //
  // ^찪q
  //
  digitalWrite(uartTx, GPIO_LEVEL_HIGH);

  //
  // Clear the buffer
  //
  memset(&(buff[0]), 0, MAX_DATA_SIZE);
  buff_index = 0;
  
  //
  // UART 
  //
  if(Hal_Uart_DataRecv(uartIdx, &data32) == 0) {
    buff[buff_index] = data32;
    delay(1);
    buff_index++;
    while(Hal_Uart_DataRecv(uartIdx, &data32) == 0) {
      if(buff_index >= MAX_DATA_SIZE) {
        break;
      }

      buff[buff_index] = data32;
      delay(3);
      buff_index++;
    }

    //
    // o{ŦX榡ƭ
    //
    if(buff[0] == 0xff) {
      //
      // p Check Sum
      //
      int sum = (buff[0] + buff[1] + buff[2]) & 0x00ff;
      if(sum == buff[3]) { // Check Sum T
        int distance = buff[1] * 256;
        distance += buff[2];
        //Serial.print(distance);
        //Serial.println(" mm");
        return distance; // Ǧ^Z
      }
    }
  }

  return 0; // Ǧ^ 0AN~C
}

/*************************************************************************
* FUNCTION:
*   Main_AppThread_1
*
* DESCRIPTION:
*   the application thread 1
*
* PARAMETERS
*   1. argu     : [In] the input argument
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_AppThread_1(void *argu)
{
    //
    // ŧiGĲoWiP GPIO
    //
    E_GpioIdx_t gpio_ultra_sonic_tri;
    T_OPL1000_Gpio *gpio_ultra_sonic_tri_ptr;

    //
    // ŧiG~q GPIO
    //
    E_GpioIdx_t gpio_relay;
    T_OPL1000_Gpio *gpio_relay_ptr;

    printf("Water level detection begins.\n");

    //
    // oWi triger GPIO 
    //
    gpio_ultra_sonic_tri_ptr = &OPL1000_periph.gpio[0];
    gpio_ultra_sonic_tri = Hal_Pinmux_GetIO(gpio_ultra_sonic_tri_ptr->pin);
    digitalWrite(gpio_ultra_sonic_tri, GPIO_LEVEL_HIGH);

    //
    // o~q GPIO 
    //
    gpio_relay_ptr = &OPL1000_periph.gpio[1];
    gpio_relay = Hal_Pinmux_GetIO(gpio_relay_ptr->pin);
    digitalWrite(gpio_relay, GPIO_LEVEL_LOW);

    //
    // 쪺Lj
    //    
    while (1) {
        //
        // WiZPܤZ (GMM)
        //
        Distance = readUltraSonicUart(UART_IDX_0, gpio_ultra_sonic_tri);
        //printf("Distance = %d\n", Distance);
        
        //
        // WiZPܦZ - WiZPܤZ = 찪
        //
        WaterLevel = SENSOR_TO_BOTTOM - ((float)Distance / 10); // G
        printf("WaterLevel = %f CM\n", WaterLevel);
        osDelay(100);

        //
        // CؼФ(C)
        //
        if (WaterLevel < TargetWaterLevelL) {
            //
            // }`
            //
            digitalWrite(gpio_relay, GPIO_LEVEL_HIGH);
            printf("Water level is too Low\n");
            FlagAlert = 5;
        }

        //
        // CؼФ()
        //
        if (WaterLevel >= TargetWaterLevelH) {
            if(digitalRead(gpio_relay) == GPIO_LEVEL_HIGH) {
                printf("=== Stop Pump ===\n");
            }

            //
            // 
            //
            digitalWrite(gpio_relay, GPIO_LEVEL_LOW);
            FlagAlert = 0;
        }
    }
}


/*************************************************************************
* FUNCTION:
*   gpio_int_test
*
* DESCRIPTION:
*   an example that generate the output pulses from one gpio to another,
*   then the input side will be triggered the interrupt.
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
/*static void gpio_int_init(void)
{
    E_GpioIdx_t gpio_idx_button;   
    T_OPL1000_Gpio *gpio_ptr_button;

    gpio_ptr_button = &OPL1000_periph.gpio[1];
    gpio_idx_button = Hal_Pinmux_GetIO(gpio_ptr_button->pin);

    Hal_Vic_GpioDirection(gpio_idx_button, GPIO_INPUT);
    Hal_Vic_GpioCallBackFuncSet(gpio_idx_button, gpio_int_callback);
    Hal_Vic_GpioIntTypeSel(gpio_idx_button, INT_TYPE_RISING_EDGE);
    Hal_Vic_GpioIntInv(gpio_idx_button, 0);
    Hal_Vic_GpioIntMask(gpio_idx_button, 0);
    Hal_Vic_GpioIntEn(gpio_idx_button, 1);
}*/

/*************************************************************************
* FUNCTION:
*   gpio_int_callback
*
* DESCRIPTION:
*   an example that generate the output pulses from one gpio to another,
*   then the input side will be triggered the interrupt.
*
* PARAMETERS
*   tGpioIdx    : Index of call-back GPIO
*
* RETURNS
*   none
*
*************************************************************************/
//
// s_ callback
//
/*static void gpio_int_callback(E_GpioIdx_t tGpioIdx)
{
    ButtonPressed = 1;
}*/

/*************************************************************************
* FUNCTION:
*   Main_AppMessageQSend
*
* DESCRIPTION:
*   send the message into AppMessageQ
*
* PARAMETERS
*   1. ulData   : [In] the data from uart rx
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_AppMessageQSend(uint32_t ulData)
{
    S_MessageQ *ptMsgPool;
    
    // allocate the memory pool
    ptMsgPool = (S_MessageQ *)osPoolCAlloc(g_tAppMemPoolId);
    if (ptMsgPool == NULL)
    {
        printf("To allocate the memory pool for AppMessageQ is fail.\n");
        goto done;
    }
    
    // copy the message content
    ptMsgPool->ulEvent = APP_EVENT_UART_RX;
    ptMsgPool->ulData = ulData;
    
    // send the message
    if (osOK != osMessagePut(g_tAppMessageQ, (uint32_t)ptMsgPool, osWaitForever))
    {
        printf("To send the message for AppMessageQ is fail.\n");
        
        // free the memory pool
        osPoolFree(g_tAppMemPoolId, ptMsgPool);
        goto done;
    }
        
done:
    return;
}

/*************************************************************************
* FUNCTION:
*   uart_int_test
*
* DESCRIPTION:
*   an example that read and write data on UART0, and send the rx data to
*   thread when the rx data is received in UART0 interrupt.
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void init_thread(void)
{
    osThreadDef_t tThreadDef;
    osMessageQDef_t tMessageDef;
    osPoolDef_t tMemPoolDef;
    
    // create the thread for AppThread_1
    tThreadDef.name = "App_1";
    tThreadDef.pthread = Main_AppThread_1;
    tThreadDef.tpriority = OS_TASK_PRIORITY_APP;        // osPriorityNormal
    tThreadDef.instances = 0;                           // reserved, it is no used
    tThreadDef.stacksize = OS_TASK_STACK_SIZE_APP;      // (512), unit: 4-byte, the size is 512*4 bytes
    g_tAppThread_1 = osThreadCreate(&tThreadDef, NULL);
    if (g_tAppThread_1 == NULL)
    {
        printf("To create the thread for AppThread_1 is fail.\n");
    }


    wifi_conn();


    // create the message queue for AppMessageQ
    tMessageDef.queue_sz = APP_MESSAGE_Q_SIZE;          // number of elements in the queue
    tMessageDef.item_sz = sizeof(S_MessageQ);           // size of an item
    tMessageDef.pool = NULL;                            // reserved, it is no used
    g_tAppMessageQ = osMessageCreate(&tMessageDef, g_tAppThread_2);
    if (g_tAppMessageQ == NULL)
    {
        printf("To create the message queue for AppMessageQ is fail.\n");
    }
    
    // create the memory pool for AppMessageQ
    tMemPoolDef.pool_sz = APP_MESSAGE_Q_SIZE;           // number of items (elements) in the pool
    tMemPoolDef.item_sz = sizeof(S_MessageQ);           // size of an item
    tMemPoolDef.pool = NULL;                            // reserved, it is no used
    g_tAppMemPoolId = osPoolCreate(&tMemPoolDef);
    if (g_tAppMemPoolId == NULL)
    {
        printf("To create the memory pool for AppMessageQ is fail.\n");
    }
    
    // set the rx callback function and enable the rx interrupt
    //Hal_Uart_RxCallBackFuncSet(UART_IDX_0, Main_AppMessageQSend);
    //Hal_Uart_RxIntEn(UART_IDX_0, 1);
}

/*************************************************************************
* FUNCTION:
*   Main_ApsUartRxDectecConfig
*
* DESCRIPTION:
*   detect the GPIO high level if APS UART Rx is connected to another UART Tx port.
*
* PARAMETERS
*   none
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_ApsUartRxDectecConfig(void)
{
    E_GpioLevel_t eGpioLevel;

    Hal_Pin_ConfigSet(1, PIN_TYPE_GPIO_INPUT, PIN_DRIVING_LOW);
    eGpioLevel = Hal_Vic_GpioInput(GPIO_IDX_01);
    if (GPIO_LEVEL_HIGH == eGpioLevel)
    {
        // it is connected
        Hal_Pin_ConfigSet(1, HAL_PIN_TYPE_IO_1, HAL_PIN_DRIVING_IO_1);
        Hal_DbgUart_RxIntEn(1);
    }
    else //if (GPIO_LEVEL_LOW == eGpioLevel)
    {
        // it is not conncected, set the high level to trigger the GPIO interrupt
        Hal_Vic_GpioCallBackFuncSet(GPIO_IDX_01, Main_ApsUartRxDectecCb);
        //Hal_Vic_GpioDirection(GPIO_IDX_01, GPIO_INPUT);
        Hal_Vic_GpioIntTypeSel(GPIO_IDX_01, INT_TYPE_LEVEL);
        Hal_Vic_GpioIntInv(GPIO_IDX_01, 0);
        Hal_Vic_GpioIntMask(GPIO_IDX_01, 0);
        Hal_Vic_GpioIntEn(GPIO_IDX_01, 1);
    }
}

/*************************************************************************
* FUNCTION:
*   Main_ApsUartRxDectecCb
*
* DESCRIPTION:
*   detect the GPIO high level if APS UART Rx is connected to another UART Tx port.
*
* PARAMETERS
*   1. tGpioIdx : Index of call-back GPIO
*
* RETURNS
*   none
*
*************************************************************************/
static void Main_ApsUartRxDectecCb(E_GpioIdx_t tGpioIdx)
{
    // disable the GPIO interrupt
    Hal_Vic_GpioIntEn(GPIO_IDX_01, 0);

    // it it connected
    Hal_Pin_ConfigSet(1, HAL_PIN_TYPE_IO_1, HAL_PIN_DRIVING_IO_1);
    Hal_DbgUart_RxIntEn(1);
}


